/*
    Парус 8 - Панели мониторинга - ТОиР - Прогнозирование технического состояния
    Закладка прогнозирования: дополнительная разметка и вёрстка клиентских элементов
*/

//---------------------
//Подключение библиотек
//---------------------

import React from "react"; //Классы React
import PropTypes from "prop-types"; //Контроль свойств компонента
import { Box, Card, CardContent, Typography, CardActions, Button } from "@mui/material"; //Интерфейсные компоненты
import { formatDateRF } from "../../core/utils"; //Вспомогательные функции

//---------
//Константы
//---------

//Стили
const STYLES = {
    TECH_OBJ_CARD_TITLE: { fontSize: 14 },
    TECH_OBJ_CARD_SUB_TITLE: { mb: 1.5 }
};

//-----------
//Тело модуля
//-----------

//Карточка технического объекта
const TechObjCard = ({ cardData }) => {
    //Генерация содержимого
    return (
        <Box p={2}>
            <Card>
                <CardContent>
                    <Typography sx={STYLES.TECH_OBJ_CARD_TITLE} color="text.secondary" gutterBottom>
                        Технический объект
                    </Typography>
                    <Typography variant="h5" component="div">
                        {cardData.SCODE}
                    </Typography>
                    <Typography sx={STYLES.TECH_OBJ_CARD_SUB_TITLE} color="text.secondary">
                        {cardData.DOPER_DATE}
                    </Typography>
                    <Typography variant="body2">{cardData.SNAME}</Typography>
                </CardContent>
                <CardActions>
                    <Button size="large">Прогнозировать</Button>
                    <Button size="large">Ремонтировать</Button>
                    <Button size="large">Обучать</Button>
                </CardActions>
            </Card>
        </Box>
    );
};

//Контроль свойств - Карточка технического объекта
TechObjCard.propTypes = {
    cardData: PropTypes.object
};

//Форматирование значений колонок таблицы технических объектов
const eqConfigTechObjTableValueFormatter = ({ value, columnDef }) => {
    switch (columnDef.name) {
        case "DOPER_DATE":
            return formatDateRF(value);
    }
    return value;
};

//----------------
//Интерфейс модуля
//----------------

export { TechObjCard, eqConfigTechObjTableValueFormatter };
